/* The box-bin element holds the button-box element which holds each window-button.
 *
 * Control spacing between each button via .button-box.
 */
.box-bin {
	border-width: 0px;
	padding: 0px;
	margin: 0px;
}

.button-box {
	spacing: 4px;
}

/* The style for each individual window button */
.box-bin .window-button {
	width: 24px;
	height: 20px;
}

/* The style for the window button when hovered over */
.box-bin .window-button:hover {
}

/* Here you can style each individual button. You need to define style classes
 * ".minimize", ".maximize" and ".close".
 * You can additionally define styles for the :hover and :active pseudo classes
 * if you wish.
 * See the 'Radiance' style.css for an example of this.
 */
/* Style for the minimize button */
.minimize {
	background-image: url('min.svg');
}

.minimize:hover {
	background-image: url('min_hover.svg');
}

.minimize:active {
	background-image: url('min_active.svg');
}

/* Style for the maximize button */
.maximize {
	background-image: url('max.svg');
}

.maximize:hover {
	background-image: url('max_hover.svg');
}

.maximize:active {
	background-image: url('max_active.svg');
}

/* Style for the close button */
.close {
	background-image: url('close.svg');
}

.close:hover {
	background-image: url('close_hover.svg');
}

.close:active {
	background-image: url('close_active.svg');
}