.button-box {
    spacing: 10px;
}

.box-bin .window-button {

    width: 12px;
    height: 12px;
    
    border-radius: 8px;
    border-width: 2px;
    border-color: rgba(0,0,0,0);
    
    background-gradient-direction: vertical;
    background-gradient-start: rgba(255,255,255,1);
    background-gradient-end: rgba(235,235,235,1);
    
    transition-duration: 100;
}

.box-bin .window-button:active {
    background-gradient-start: rgba(207,207,207,1);
    background-gradient-end: rgba(249,249,249,1);
}

.minimize {
    
}

.minimize:hover {
    border-color: rgba(209,174,51,0.75);
}

.minimize:active {
    border-color: rgba(202,169,49,0.75);
}

.maximize {
    
}

.maximize:hover {
    border-color: rgba(80,209,51,0.75);
}

.maximize:active {
    border-color: rgba(85,179,64,0.75);
}

.close {
    
}

.close:hover {
    border-color: rgba(189,46,46,0.75);
}

.close:active {
    border-color: rgba(209,51,51,1);
}
